@@ -21,7 +21,7 @@ import glob
 from pint_pal.utils import write_if_changed, apply_cut_flag, apply_cut_select
 from pint_pal.lite_utils import new_changelog_entry
 from pint_pal.lite_utils import check_toa_version, check_tobs
-from pint_pal.defaults import *
+import pint_pal.config
 
 class TimingConfiguration:
     """
@@ -45,13 +45,51 @@ class TimingConfiguration:
         tim_directory (optional) : override the tim directory specified in the config
         par_directory (optional) : override the par directory specified in the config
         """
-        self.filename = filename
-        with open(filename) as FILE:
+        self.filename = os.path.realpath(os.path.expanduser(filename))
+        with open(self.filename) as FILE:
             self.config = yaml.load(FILE, Loader=yaml.FullLoader)
-        self.tim_directory = self.config['tim-directory'] if tim_directory is None else tim_directory
-        self.par_directory = self.config['par-directory'] if par_directory is None else par_directory
+        if tim_directory is not None:
+            self.config['tim-directory'] = tim_directory
+        if par_directory is not None:
+            self.config['par-directory'] = par_directory
         self.skip_check = self.config['skip-check'] if 'skip-check' in self.config.keys() else ''
 
+    @property
+    def tim_directory(self):
+        """
+        Location of tim files, as specified in the config.
+        This returns the absolute path to the tim directory.
+        """
+        return os.path.realpath(
+            os.path.join(pint_pal.config.DATA_ROOT, self.config['tim-directory'])
+        )
+
+    @tim_directory.setter
+    def tim_directory(self, tim_directory):
+        """
+        Set tim directory.
+        If a relative path is supplied, it will be turned into an absolute path.
+        """
+        self.config['tim-directory'] = tim_directory
+
+    @property
+    def par_directory(self):
+        """
+        Location of par files, as specified in the config.
+        This returns the absolute path to the par directory.
+        """
+        return os.path.realpath(
+            os.path.join(pint_pal.config.DATA_ROOT, self.config['par-directory'])
+        )
+
+    @par_directory.setter
+    def par_directory(self, par_directory):
+        """
+        Set par directory.
+        If a relative path is supplied, it will be turned into an absolute path.
+        """
+        self.config['par-directory'] = par_directory
+
     def get_source(self):
         """ Return the source name """
         return self.config['source']
@@ -119,9 +157,9 @@ class TimingConfiguration:
         m = model.get_model(par_path,allow_name_mixing=True)
         match = re.search(r"#\s+Created:\s+(\S+)", open(par_path).read())
         if match:
-            m.created_time = match.group(1)
-            log.info(f"Par file created: {m.created_time}")
-        m.file_mtime = Time(os.path.getmtime(par_path), format="unix").isot
+            m.meta['created_time'] = match.group(1)
+            log.info(f"Par file created: {m.meta['created_time']}")
+        m.meta['file_mtime'] = Time(os.path.getmtime(par_path), format="unix").isot
 
 
         if m.PSR.value != self.get_source():
@@ -133,7 +171,7 @@ class TimingConfiguration:
                           usepickle=usepickle,
                           bipm_version=BIPM,
                           ephem=EPHEM,
-                          planets=PLANET_SHAPIRO,
+                          planets=pint_pal.config.PLANET_SHAPIRO,
                           model=m,
                           picklefilename=picklefilename,
                           include_pn=include_pn
@@ -336,8 +374,11 @@ class TimingConfiguration:
     def construct_fitter(self, to, mo):
         """ Return the fitter, tracking pulse numbers if available """
         fitter_name = self.config['fitter']
-        fitter_class = getattr(pint.fitter, fitter_name)
-        return fitter_class(to, mo)
+        if fitter_name == 'Auto':
+            return pint.fitter.Fitter.auto(to, mo)
+        else:
+            fitter_class = getattr(pint.fitter, fitter_name)
+            return fitter_class(to, mo)
 
     def get_toa_type(self):
         """ Return the toa-type string """
@@ -359,7 +400,7 @@ class TimingConfiguration:
 
     def get_excised(self):
         """ Return excised-tim file if set and exists"""
-        if 'excised-tim' in self.config['intermediate-results'].keys():
+        if 'excised-tim' in self.config['intermediate-results'].keys() and self.config['intermediate-results']['excised-tim']:
             if os.path.exists(self.config['intermediate-results']['excised-tim']):
                 return self.config['intermediate-results']['excised-tim']
         return None
@@ -381,6 +422,12 @@ class TimingConfiguration:
         if 'orphaned-rec' in self.config['ignore'].keys():
             return self.config['ignore']['orphaned-rec']
         return None 
+
+    def get_bad_group(self):
+        """Return bad group(s)"""
+        if 'bad-group' in self.config['ignore'].keys():
+            return self.config['ignore']['bad-group']
+        return None
     
     def get_poor_febes(self):
         """Return poor frontend/backend combinations for removal"""
@@ -635,13 +682,13 @@ class TimingConfiguration:
         """ Return desired frequency ratio """
         if 'fratio' in self.config['dmx'].keys():
             return self.config['dmx']['fratio']
-        return FREQUENCY_RATIO
+        return pint_pal.config.FREQUENCY_RATIO
 
     def get_sw_delay(self):
         """ Return desired max(solar wind delay) threshold """
         if 'max-sw-delay' in self.config['dmx'].keys():
             return self.config['dmx']['max-sw-delay']
-        return MAX_SOLARWIND_DELAY
+        return pint_pal.config.MAX_SOLARWIND_DELAY
 
     def get_custom_dmx(self):
         """ Return MJD/binning params for handling DM events, etc. """
@@ -682,7 +729,7 @@ class TimingConfiguration:
     def apply_ignore(self,toas,specify_keys=None,warn=False,model=None):
         """ Basic checks and return TOA excision info. """
         OPTIONAL_KEYS = ['mjd-start','mjd-end','snr-cut','bad-toa', 'bad-toa-averaged','bad-range','bad-file',
-                        'orphaned-rec','prob-outlier','poor-febe','orb-phase-range']
+                        'orphaned-rec','bad-group','prob-outlier','poor-febe','orb-phase-range']
         EXISTING_KEYS = self.config['ignore'].keys()
         VALUED_KEYS = [k for k in EXISTING_KEYS if self.config['ignore'][k] is not None]
 
@@ -710,10 +757,15 @@ class TimingConfiguration:
         # All info here about selecting various TOAs.
         # Select TOAs to cut, then use apply_cut_flag.
         if 'orphaned-rec' in valid_valued:
-            fs = np.array([f['f'] for f in toas.orig_table['flags']])
+            fs = np.array([(f['f'] if 'f' in f else None) for f in toas.orig_table['flags']])
             for o in self.get_orphaned_rec():
                 orphinds = np.where(fs==o)[0]
                 apply_cut_flag(toas,orphinds,'orphaned',warn=warn)
+        if 'bad-group' in valid_valued:
+            for flag, value in self.get_bad_group():
+                vals = np.array([f[flag] for f in toas.orig_table['flags'] if flag in list(f.keys())])
+                bad_inds = np.where(vals==value)[0]
+                apply_cut_flag(toas,bad_inds,'badgroup',warn=warn)
         if 'mjd-start' in valid_valued:
             mjds = np.array([m for m in toas.orig_table['mjd_float']])
             startinds = np.where(mjds < self.get_mjd_start())[0]
@@ -723,7 +775,9 @@ class TimingConfiguration:
             endinds = np.where(mjds > self.get_mjd_end())[0]
             apply_cut_flag(toas,endinds,'mjdend',warn=warn)
         if 'snr-cut' in valid_valued:
-            snrs = np.array([float(f['snr']) for f in toas.orig_table['flags']])
+            if any('snr' not in f for f in toas.orig_table['flags']):
+                log.warning("Excision key 'snr-cut' in use, but some TOAs are missing the 'snr' flag.")
+            snrs = np.array([(float(f['snr']) if 'snr' in f else np.nan) for f in toas.orig_table['flags']])
             snrinds = np.where(snrs < self.get_snr_cut())[0]
             apply_cut_flag(toas,snrinds,'snr',warn=warn)
             if self.get_snr_cut() > 8.0 and self.get_toa_type() == 'NB':
@@ -731,7 +785,7 @@ class TimingConfiguration:
             if self.get_snr_cut() > 25.0 and self.get_toa_type() == 'WB':
                 log.warning('snr-cut should be set to 25; try excising TOAs using other methods.')
         if 'poor-febe' in valid_valued:
-            fs = np.array([f['f'] for f in toas.orig_table['flags']])
+            fs = np.array([(f['f'] if 'f' in f else None) for f in toas.orig_table['flags']])
             for febe in self.get_poor_febes():
                 febeinds = np.where(fs==febe)[0]
                 apply_cut_flag(toas,febeinds,'poorfebe',warn=warn)
@@ -788,7 +842,7 @@ class TimingConfiguration:
                 log.warning(f'One or more bad-file entries lack reasons for excision; please add them.')
         if 'bad-range' in valid_valued:
             mjds = np.array([m for m in toas.orig_table['mjd_float']])
-            backends = np.array([f['be'] for f in toas.orig_table['flags']])
+            backends = np.array([(f['be'] if 'be' in f else None) for f in toas.orig_table['flags']])
             for br in self.get_bad_ranges():
                 if len(br) > 2:
                     rangeinds = np.where((mjds>br[0]) & (mjds<br[1]) & (backends==br[2]))[0]
@@ -808,8 +862,9 @@ class TimingConfiguration:
         if 'bad-toa' in valid_valued:
             logwarntoa = False
             names = np.array([f['name'] for f in toas.orig_table['flags']])
-            subints = np.array([int(f['subint']) for f in toas.orig_table['flags']])
-            if self.get_toa_type() == 'NB': chans = np.array([int(f['chan']) for f in toas.orig_table['flags']])
+            subints = np.array([(int(f['subint']) if 'subint' in f else None) for f in toas.orig_table['flags']])
+            if self.get_toa_type() == 'NB':
+                chans = np.array([(int(f['chan']) if 'chan' in f else None) for f in toas.orig_table['flags']])
             btinds = []
             for bt in self.get_bad_toas():
                 if len(bt) < 4: logwarntoa = True
