@@ -42,8 +42,12 @@ def whiten_resids(fitter, restype = 'postfit'):
         rs = fitter['time_resids']
         noise_rs = fitter['noise_resids']
         # Now check if red noise residuals
-        if "pl_red_noise" in noise_rs:
+        if "pl_red_noise" and "pl_DM_noise" in noise_rs:
+            wres = rs - noise_rs['pl_red_noise'] - noise_rs['pl_DM_noise']
+        elif "pl_red_noise" in noise_rs:
             wres = rs - noise_rs['pl_red_noise']
+        elif "pl_DM_noise" in noise_rs:
+            wres = rs - noise_rs['pl_DM_noise']
         else:
             log.warning("No red noise, residuals already white. Returning input residuals...")
             wres = rs
@@ -69,8 +73,12 @@ def whiten_resids(fitter, restype = 'postfit'):
             # Get number of residuals
         num_res = len(time_resids)
         # Check that the key is in the dictionary
-        if "pl_red_noise" in noise_resids:
+        if "pl_red_noise" and "pl_DM_noise" in noise_resids:
+            wres = time_resids - noise_resids['pl_red_noise'][:num_res] - noise_resids['pl_DM_noise'][:num_res]
+        elif "pl_red_noise" in noise_resids:
             wres = time_resids - noise_resids['pl_red_noise'][:num_res]
+        elif "pl_DM_noise" in noise_resids:
+            wres = time_resids - noise_resids['pl_DM_noise'][:num_res]
         else:
             log.warning("No red noise, residuals already white. Returning input residuals...")
             wres = time_resids
@@ -558,7 +566,11 @@ def pdf_writer(fitter,
     # print list of tim file names, limit two tim files per line
     fsum.write(r'Input tim files:' + "\n")
     fsum.write(r'\begin{itemize}' + "\n")
-    for tf in fitter.toas.filename:
+    if isinstance(fitter.toas.filename, str):
+        tim_files = [fitter.toas.filename]
+    else:
+        tim_files = fitter.toas.filename
+    for tf in tim_files:
         fsum.write(r'\item ' + verb(tf.split('/')[-1]) + '\n')
     fsum.write(r'\end{itemize}' + "\n")
     fsum.write('Span: %.1f years (%.1f -- %.1f)\\\\\n ' % (span/365.24,
@@ -1276,3 +1288,64 @@ def check_recentness_excision(tc):
     if used_excision != available_excision[-1]:
         log.warning(f"Using excision from {used_excision} but {available_excision[-1]} is available")
     return used_excision, available_excision
+
+
+def no_ecorr_average(toas, resids, use_noise_model=True):
+        """Create a dictionary to match those created by ecorr_avg for TOAs w/no-ecorr -- 
+        to be used with EPTA data for consistent IPTA averaged plots. 
+        
+        Use a toa object and residaul object as input as input instead of just residual object (oh well)
+        Also tell it pre/post fit.
+        
+        Arguments
+        ---------
+        toas[object]: PINT TOA object.
+        resids[object]: PINT residuals object
+        use_noise_model[boolean]: Default true.
+        
+        Returns a dictionary with the following entries:
+
+          mjds           Average MJD for each segment
+
+          freqs          Average topocentric frequency for each segment
+
+          time_resids    Average residual for each asegment, time units
+
+          noise_resids   Dictionary of per-noise-component average residual
+
+          errors         Uncertainty on averaged residuals
+
+          indices        List of lists giving the indices of TOAs in the original TOA table for each segment
+        """
+        
+        # We already know that TOAs do not have ecorr. Identify the indices where that's true.
+        inds= np.where(toas['pta']=='EPTA')[0]
+        
+        only_epta_toas = toas[inds]
+        
+        no_avg = {}
+        no_avg["mjds"] = only_epta_toas.get_mjds()
+
+        if use_noise_model:
+            err = resids.model.scaled_toa_uncertainty(only_epta_toas)
+        else:
+            err = only_epta_toas.get_errors()
+
+        # Weighted average of various quantities
+        no_avg = {}
+        no_avg["mjds"] = only_epta_toas.get_mjds()
+        no_avg["freqs"] = only_epta_toas.get_freqs()
+        no_avg["time_resids"] = resids.time_resids[inds]
+        no_avg["noise_resids"] = {}
+        for k in resids.noise_resids.keys():
+            no_avg["noise_resids"][k] = resids.noise_resids[k][inds]
+
+        # Uncertainties
+        no_avg["errors"] = err
+        
+        # Indices back into original TOA list
+        no_avg["indices"] = inds
+        
+
+        return no_avg
+    
\ No newline at end of file
